/*
 * $Id: BaseTransactionFactory.java,v 1.1 2010/01/31 00:41:59 srevilak Exp $
 */

package edu.umb.bench.framework;

/**
 * Base class for transaction factories.
 *
 * @version $Revision: 1.1 $ $Date: 2010/01/31 00:41:59 $
 */
public abstract class BaseTransactionFactory<T>
    extends BaseObject
    implements ITransactionFactory<T> {

    private double mScale;
    
    
    /**
     * The default init function is a no-op.  This is meant as a convenience
     * for factories that don't happen to need an initialization method.
     */
    public void init() {
        /* no-op */
    }

    /**
     * Sets the scale for this workload run.  Scale
     * is typically used to complete parameters for transactions.
     */
    public void setScale(double scale) {
        if (scale <= 0.0) {
            throw new IllegalArgumentException("non-positive scale: " + scale);
        }
        mScale = scale;
    }


    /**
     * Returns the workload scale
     */
    public double getScale() {
        return mScale;
    }
      

    /**
     * Performs common setup for a transaction object.
     * Common setup insists of propagating objects
     * specified via IConnectLog.
     */
    protected void commonSetup(ITransaction<T> tx) {
        assert tx != null;
        if (tx != null) {
            tx.setConnection(getConnection());
            tx.setLogger(getLogger());
            tx.setConfig(getConfig());
        }
    }

}
