/*
 * $Id: IAuditor.java,v 1.3 2010/02/08 21:14:10 srevilak Exp $
 */

package edu.umb.bench.framework;

import java.util.Map;
import java.sql.SQLException;

/**
 * Auditor interface.  The auditor is used in two places:
 *
 * <ul>
 * <li>After setup, to take any preliminary measurements on the 
 * table data.
 * <li>after test completion, to take any final measurements on 
 * the test data.
 * </ul>
 *
 * Auditors are expected to use a simple data representation:
 * a map of strings to objects (these objects being strings or numbers).
 *
 * Benchmarks can expect the same auditor to be used for
 * pre-and post measurements.  The auditor may or may not be
 * given the same connection object.
 *
 * <p> Finally, there are two types of auditor: one that is benchmark
 * specific, and another that is database system specific.  The former
 * is particular to a single benchmark.  The latter might be specific
 * to MySQL or Oracle, etc.  The auditing order is
 *
 * <ul>
 * <li>benchmark postSetup
 * <li>system postSetup
 * <li>system postTest
 * <li>benchmark postTest
 * </ul>
 *
 * @version $Revision: 1.3 $ $Date: 2010/02/08 21:14:10 $
 * @see ISystemAuditor
 */
public interface IAuditor extends IConnectLog {

    /**
     * Examines the database after setup, but before any warmup or measurement
     * has been done.
     */
    public void postSetup() throws SQLException;

    /**
     * Examines the database after tests have 
     * been run.
     */
    public void postTest() throws SQLException;

    /**
     * Retrieves the auditors results
     */
    public Map<String,Object> getResults();
    
}
