/*
 * $Id: IErrorInterpreter.java,v 1.4 2010/07/01 02:10:51 srevilak Exp $
 */

package edu.umb.bench.framework;

import java.sql.SQLException;

/**
 * Interface for objects that interpret sql errors.
 *
 * @version $Revision: 1.4 $ $Date: 2010/07/01 02:10:51 $
 */
public interface IErrorInterpreter {

    /**
     * Error Types
     */
    enum ErrorType { 
        /**
         * Not an error
         */
        NONE,

        /** 
         * A non-recoverable error.  This should cause termination of
         * a test thread
         */
        TERMINAL,

            /**
             * A concurrent write/write conflict error.  We use "FCW"
             * (First committer wins) as a generic name for write
             * write conflicts, although the actual strategy might be
             * first updater wins, second updater looses, etc.
             */
            FCW,

            /**
             * A deadlock; two transactions are waiting for each other
             */
            DEADLOCK,

            /** 
             * A cycle, or other dependency structure that indicates
             * non-serializable execution.  We use the generic name
             * "cycle", but depending on the dbms configuration, it could
             * mean dangerous structure.
             */
            CYCLE,

            /**
             * A timeout.
             */
            TIMEOUT,

            /**
             * CTG path too long
             */
            PATH_TOO_LONG,

            /**
             * An unknown error.  Probably terminal.
             */
            UNKNOWN;
    }


    /**
     * Given an SQL Error, return TRUE if the error
     * should cause termination of the test thread.  Return false
     * if the test thread can continue
     */
    public ErrorType classifyError(SQLException e);

}
