/*
 * $Id: IForeman.java,v 1.2 2011/06/04 00:40:27 srevilak Exp $
 */

package edu.umb.bench.framework;

import java.util.List;

/**
 * An interface for the foreman; the foreman directs execution of workers.
 *
 * @author srevilak
 * @since 6/2/2011
 */
public interface IForeman {

    /**
     * Provides the foreman with a connection manager.
     */
    public void setConnectionManager(ConnectionManager cm);

    /**
     * Provides the foreman with access to the configuration object.
     */
    public void setConfig(BenchConfig config);

    /**
     * Provides the forman with a logger
     */
    public void setLogger(ILogger logger);


    /**
     * Initialization.  init() tells the foreman to set up (but not run)
     * the workers
     */
    public void init();

    /**
     * Starts the workers
     */
    public void start();


    /**
     * This method waits until all workers are finished.
     */
    public void shutdown();

    /**
     * Returns the list of worker objects.
     */
    public List<IWorker> getWorkers();

    /**
     * Return the number of client threads connected to the database
     */
    public int getClientCount();

}

