/*
 * $Id: ISetup.java,v 1.3 2010/05/17 17:18:21 srevilak Exp $
 */

package edu.umb.bench.framework;

import java.sql.SQLException;
import java.sql.Connection;

/**
 * Interface to set up for a benchmark run.  An implmenting class is
 * responsible for (a) (re)creating the schema for the benchmark and
 * (b) populating the tables in the schema
 *
 * @version $Revision: 1.3 $ $Date: 2010/05/17 17:18:21 $
 */
public interface ISetup extends IConnectLog {

    /**
     * Sets the scale for this benchmark.
     */
    public void setScale(double scale);


    /**
     * Creates tables needed for this benchmark.
     *
     * @exception SQLException if a database error occurs.
     */
    public void createSchema() throws SQLException;


    /**
     * Loads tables for this benchmark test.
     *
     * @exception SQLException if an error occurs during load.
     */
    public void fillTables() throws SQLException;


    /**
     * Performs any warmup necessary for this benchmark test.
     *
     * <p>Note: warmup is always executed, even if the setup portion
     * of a test is disabled.
     */
    public void warmup() throws SQLException;
}
