/*
 * $Id: ISystemAuditor.java,v 1.1 2010/02/08 21:11:34 srevilak Exp $
 */

package edu.umb.bench.framework;

import java.util.Map;
import java.sql.SQLException;

/**
 * System Auditor interface.
 *
 * <p>The System Auditor's job is to record database configuration
 * parameters and database performance counters.
 *
 * @version $Revision: 1.1 $ $Date: 2010/02/08 21:11:34 $
 * @see IAuditor
 */
public interface ISystemAuditor extends IConnectLog {

    /**
     * Examines the database after setup, but before any warmup or measurement
     * has been done.
     */
    public void postSetup() throws SQLException;

    /**
     * Examines the database after tests have 
     * been run.
     */
    public void postTest() throws SQLException;

    /**
     * Retrieves the database configuration.
     *
     * Auditors should return at least an empty map.
     */
    public Map<String,Object> getConfiguration();
    

    /**
     * Retrieves the value of any database performance counters.
     *
     * Typically, these counter values will be deltas
     * between postSetup and postTest
     */
    public Map<String,Object> getCounters();
}
