/*
 * $Id: ITransaction.java,v 1.4 2011/06/03 01:16:05 srevilak Exp $
 */

package edu.umb.bench.framework;
import java.sql.SQLException;

/**
 * Transaction interface.
 *
 * <p> Transactions are instantiations of a particular workload
 * transaction profile.  They are created and configured by 
 * a TransactionFactory.
 *
 * @version $Revision: 1.4 $ $Date: 2011/06/03 01:16:05 $
 * @see ITransactionFactory
 */
public interface ITransaction<T> extends IConnectLog {

    /**
     * Executes the transaction.
     *
     * @param warmup indicates whether the transaction should run
     * itself in warmup mode, or measurement mode.  In warmup mode,
     * transactions should attempt to make any writes idempotent.
     */
    public void execute(boolean warmup) throws SQLException;


    /**
     * Asks this transaction to commit itself
     */
    public void commit() throws SQLException;


    /**
     * Asks the transaction to roll itself back.
     *
     * <p>Transactions are allowed to propagate SQLExceptions.
     * In the event of an exception, the framework will take care of 
     * rollback.
     *
     * <p>Transactions are expected to clean up any statements and result sets
     * that the open (e.g., in a <tt>finally</tt> clause).
     */
    public void rollback();

    
    /**
     * Returns the results of executing this transaction.  The 
     * specific type of object returned depends on the benchmark.
     */
    public T getResults();
}
