/*
 * $Id: MySQLErrorInterpreter.java,v 1.7 2010/07/01 02:10:51 srevilak Exp $
 */

package edu.umb.bench.framework;


import edu.umb.bench.framework.IErrorInterpreter.ErrorType;
import java.sql.SQLException;

/**
 * Interpreter for MySQL errors
 *
 * @version $Revision: 1.7 $ $Date: 2010/07/01 02:10:51 $
 */
public class MySQLErrorInterpreter implements IErrorInterpreter {

    /**
     * @inheritDoc
     *
     * <p>Here are examples of non-terminal errors
     *
     * <pre>
     * com.mysql.jdbc.exceptions.MySQLTransactionRollbackException: 
     *   Deadlock found when trying to get lock; try restarting transaction
     *
     * java.sql.SQLException: 
     *   Lock wait timeout exceeded; try restarting transaction
     </pre>
     */
    public ErrorType classifyError(SQLException e) {
        if (e == null) {
            return ErrorType.NONE;
        }

        String message = e.getMessage();
        if (message == null) {
            return ErrorType.UNKNOWN;
        }

        if (message.indexOf("Deadlock") >= 0) {
            return ErrorType.DEADLOCK;
        }
        
        else if (message.indexOf("Lock wait timeout") >= 0) {
            return ErrorType.TIMEOUT;
        }

        else if (message.indexOf("FCW violation") >= 0) {
            return ErrorType.FCW;
        }

        else if (message.indexOf("error 149 during COMMIT") >= 0) {
            /* cycle */
            return ErrorType.CYCLE;
        }
                                                            
        else if (message.indexOf("error 147 during COMMIT") >= 0) {
            /* CTG path too long */
            return ErrorType.PATH_TOO_LONG;
        }
                                                            
        else {
            return ErrorType.TERMINAL;
        }
    }

}
