/*
 * $Id: SimpleLogger.java,v 1.2 2010/02/08 19:17:57 srevilak Exp $
 */

package edu.umb.bench.framework;

/**
 * A simple implementation of the ILogger interface.  This logger
 * just writes to stderr.
 *
 * @see ILogger
 */
public class SimpleLogger implements ILogger {

    public static final int LEVEL_ALL = 0;
    public static final int LEVEL_INFO = 10;
    public static final int LEVEL_WARN = 20;
    public static final int LEVEL_ERROR = 30;
    public static final int LEVEL_NONE = 100;

    private int mLevel;

    public SimpleLogger() {
        this(LEVEL_INFO);
    }


    /**
     * Constructs a logger with the specified LEVEL as log mask.
     *
     * Only messsages &ge; level will be logged.
     */
    public SimpleLogger(int level) {
        mLevel = level;
    }


    public void setLevel(int level) {
        mLevel = level;
    }

    public void info(String msg) {
        logInternal(LEVEL_INFO, "INFO", msg);
    }

    
    public void warn(String msg) {
        logInternal(LEVEL_WARN, "WARN", msg);
    }

    public void error(String msg) {
        logInternal(LEVEL_ERROR, "ERROR", msg);
    }

    public void error(String msg, Throwable t) {
        logInternal(LEVEL_ERROR, "ERROR", msg, t);
    }

    
    private void logInternal(int level, String tag, String msg) {
        logInternal(level, tag, msg, null);
    }


    private void logInternal(int level, String tag, String msg,
                             Throwable e) {
        if (mLevel > level) {
            return;
        }
        String threadName = Thread.currentThread().getName();
        StringBuilder sb = new StringBuilder(2045);
        sb.append(threadName);
        sb.append(" [");
        sb.append(tag);
        sb.append("] ");
        sb.append(msg);
        System.err.println(sb.toString());
        if (e != null) {
            e.printStackTrace();
        }
    }

}
