/*
 * $Id: Card.java,v 1.3 2010/04/22 18:25:11 srevilak Exp $
 */

package edu.umb.bench.hedge;

/**
 * Table of column cardinalities
 *
 */
class Card {

    public static final int K1 = 0;
    public static final int K2 = 1;
    public static final int K4 = 2;
    public static final int K8 = 3;
    public static final int K16 = 4;
    public static final int K32 = 5;

    public static final int K_COUNT = 6;

    private String[] colNames;
    private int[] nrows;
    private static Card instance = null;

    /**
     * Creates a new Cardinality table.  Each k<sub>i</sub> denotes the
     * number of values in column k<sub>i</sub>.
     */
    private Card(int k1, int k2, int k4, int k8, int k16, int k32) {
        nrows = new int[6];
        nrows[K1] = k1;
        nrows[K2] = k2;
        nrows[K4] = k4;
        nrows[K8] = k8;
        nrows[K16] = k16;
        nrows[K32] = k32;

        colNames = new String[] {
            "k1", "k2", "k4", "k8", "k16", "k32"
        };
    }


    /**
     * Initializes the cardinality table
     */
    public static synchronized void initCard(int k1, 
                                             int k2, int k4, 
                                             int k8, int k16, 
                                             int k32) {
        if (instance == null) {
            Card tmp = new Card(k1, k2, k4, k8, k16, k32);
            instance = tmp;
        }
    }


    /**
     * Retrieves the cardinality table.  The table must have
     * already been initialized with {@link initCard}.
     */
    public static Card getInstance() {
        if (instance == null) {
            throw new IllegalStateException("Card not initialized");
        }
        return instance;
    }


    public static boolean isInitialized() {
        return (instance != null);
    }

    
    /**
     * Get the cardinality for column <em>k</em>.
     *
     * <p><tt>t</tt> should be one of the <tt>Kn</tt> constants
     * defined in this class.
     */
    public int getCard(int k) {
        assert(isValidCard(k));
        return nrows[k];
    }


    /**
     * Returns the column name for column <em>k</em>
     */
    public String getName(int k) {
        assert(isValidCard(k));
        return colNames[k];
    }


    /**
     * Returns the ordinal position of column named <em>Kn</em>
     *
     * @return -1 if k is not a Kn column.
     */
    public int getPosition(String k) {
        for (int iter = 0; iter < colNames.length; iter++) {
            if (colNames[iter].equals(k)) {
                return iter;
            }
        }
        return -1;
    }
    

    public static boolean isValidCard(int k) {
        return (k >= K1 && k <= K32);
    }

}
