/*
 * $Id: HedgeAuditor.java,v 1.2 2010/03/08 20:02:06 srevilak Exp $
 */

package edu.umb.bench.hedge;

import java.sql.SQLException;
import java.util.Map;
import java.util.HashMap;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.Connection;

import edu.umb.bench.framework.BaseAuditor;


/**
 * Auditor for the hedging benchmark.  Each audit method notes the sum
 * of the <tt>value</tt> column.
 */
public class HedgeAuditor extends BaseAuditor {

    private Map<String,Object> mData;

    public HedgeAuditor() {
        mData = new HashMap<String,Object>();
    }

    public void postSetup() throws SQLException {
        getLogger().info("post setup audit");

        commonAudit(Constants.AUDIT_VALUE_BEFORE);
    }

    public void postTest() throws SQLException {
        getLogger().info("post test audit");
        
        commonAudit(Constants.AUDIT_VALUE_AFTER);
    }


    private void commonAudit(String key) throws SQLException {
        double sum = sumValue();
        mData.put(key, Double.valueOf(sum));
    }

    private double sumValue() throws SQLException {
        Statement st = null;
        ResultSet rs = null;
        Connection conn = getConnection();
        double sum = 0;

        try {

            st = conn.createStatement();
            st.execute("select sum(value) s_value from hedge");
            rs = st.getResultSet();

            while (rs.next()) {
                sum = rs.getDouble("s_value");
            }
            rs.close();
            conn.commit();
        }

        finally {
            if (st != null) {
                st.close();
            }
        }

        return sum;
    }

    public Map<String,Object> getResults() {
        return mData;
    }
    
}
