/*
 * $Id: HedgeSetup.java,v 1.1 2010/03/08 04:00:23 srevilak Exp $
 */

package edu.umb.bench.hedge;

import java.sql.SQLException;
import edu.umb.bench.framework.BaseSetup;
import edu.umb.bench.framework.ILogger;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.Statement;
import edu.umb.bench.framework.INumberGenerator;
import edu.umb.bench.framework.GeneratorUnif;
import java.util.Properties;

/**
 * Setup class for hedge benchmark.
 *
 * @version $Revision: 1.1 $ $Date: 2010/03/08 04:00:23 $
 */
public class HedgeSetup extends BaseSetup {


    public static final String DROP_SQL = 
        "drop table if exists hedge";

    public static final String CREATE_SQL = ""
        + "create table hedge ("
        + " k1 integer not null primary key, "
        + " k2 integer not null, "
        + " k4 integer not null, "
        + " k8 integer not null, "
        + " k16 integer not null, "
        + " k32 integer not null, "
        + " value decimal(18,4) null, "
        + " pad varchar(72), "
        + "index k2_idx(k2), "
        + "index k4_idx(k4), "
        + "index k8_idx(k8), "
        + "index k16_idx(k16), "
        + "index k32_idx(k32) "
        + ") ENGINE=InnoDB";
    
    private INumberGenerator gen;


    public HedgeSetup() {
        super();
        gen = new GeneratorUnif();
    }


    public void createSchema() throws SQLException {

        Connection conn = getConnection();
        ILogger log = getLogger();
        Statement st = null;
        log.info("creating hedge table");

        try {
            st = conn.createStatement();
            st.execute(DROP_SQL);
            st.execute(CREATE_SQL);
            conn.commit();
        }
        finally {
            if (st != null) {
                st.close();
            }
        }
    }


    public void fillTables() throws SQLException {
        
        Connection conn = getConnection();
        ILogger log = getLogger();
        PreparedStatement ps = null;

        int nrows = (int) (getScale() * Constants.ROWS_PER_SCALE);

        Properties config = getConfig();
        config.put(Constants.KEY_NUM_ROWS, Integer.toString(nrows));

        log.info("filling hedge");

        Card.initCard(nrows, 
                      nrows / 2,
                      nrows / 4, 
                      nrows / 8,
                      nrows / 16,
                      nrows / 32);

        Card card = Card.getInstance();

        try {
            ps = conn.prepareStatement
                ("insert into hedge "
                 + "(k1, k2, k4, k8, k16, k32, value, pad) " 
                 + "values (?,?,?,?,?,?,?,?)");

            for (int iter = 1; iter <= nrows; iter++) {

                int k1 = iter;
                int k2 = (int) (card.getCard(Card.K2) * gen.getNumber());
                int k4 = (int) (card.getCard(Card.K4) * gen.getNumber());
                int k8 = (int) (card.getCard(Card.K8) * gen.getNumber());
                int k16 = (int) (card.getCard(Card.K16) * gen.getNumber());
                int k32 = (int) (card.getCard(Card.K32) * gen.getNumber());
                int val = 100;
                String pad = makePad();
                
                int index = 1;
                ps.setInt(index++, k1);
                ps.setInt(index++, k2);
                ps.setInt(index++, k4);
                ps.setInt(index++, k8);
                ps.setInt(index++, k16);
                ps.setInt(index++, k32);
                ps.setInt(index++, val);
                ps.setString(index++, pad);

                ps.execute();
            }

            conn.commit();
        }

        finally {
            if (ps != null) {
                ps.close();
            }
        }
    }


    private String makePad() {
        final int padlen = 72;

        int nchars = 'z' - 'a';

        StringBuilder sb = new StringBuilder(padlen);
        for (int iter = 0; iter < padlen; iter++) {
            sb.append((char) ((gen.getNumber() * nchars) + 'a'));
        }
        return sb.toString();
    }

}
