/*
 * $Id: HedgeTransaction.java,v 1.3 2011/06/03 01:16:05 srevilak Exp $
 */

package edu.umb.bench.hedge;

import edu.umb.bench.framework.BaseTransaction;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;


/**
 * Hedging transaction.
 *
 * @version $Revision: 1.3 $ $Date: 2011/06/03 01:16:05 $
 */
public class HedgeTransaction extends BaseTransaction<HedgeResult> {
    
    private HedgeResult mResult;

    private String mSelectColumn;
    private int mSelectKey;
    private int mUpdateKey;
    private int mSelectColumnNumber;

    public HedgeTransaction() {
        super();
        mResult = new HedgeResult();
    }

    public void setParams(String selectColumn, 
                          int selectColumnNumber,
                          int selectKey,
                          int updateKey) {
        mSelectColumn = selectColumn;
        mSelectColumnNumber = selectColumnNumber;
        mSelectKey = selectKey;
        mUpdateKey = updateKey;
    }


    public void execute(boolean warmup) throws SQLException {
        PreparedStatement p1 = null;
        PreparedStatement p2 = null;
        Connection conn = getConnection();
        ResultSet rs = null;

        mResult.reset();
        mResult.setColumn(mSelectColumnNumber);

        double multiplier = 0.0;

        if (! warmup) {
            multiplier = 0.001;
        }

        try {
            double sum = 0.0;
            
            p1 = conn.prepareStatement
                ("select sum(value) v from hedge where " 
                 + mSelectColumn + " =  ?");

            p1.setInt(1, mSelectKey);
            p1.executeQuery();
            rs = p1.getResultSet();

            sum = readRs(rs);
            p1.close();
            
            p2 = conn.prepareStatement
                ("update hedge set value = value + (? * ?) where k1 = ?");

            p2.setDouble(1, sum);
            p2.setDouble(2, multiplier);
            p2.setInt(3, mUpdateKey);
            int rv = p2.executeUpdate();
            assert(rv == 1);

        }

        finally {

            if (p1 != null) {
                p1.close();
            }

            if (p2 != null) {
                p2.close();
            }
        }

    }


    private double readRs(ResultSet rs) throws SQLException {
        assert rs != null;
        double sum = 0.0;

        try {
            while (rs.next()) {
                sum += rs.getDouble("v");
            }
        }

        finally {
            rs.close();
        }

        return sum;
    }


    public HedgeResult getResults() {
        return mResult;
    }

}
