/*
 * $Id: HedgeTransactionFactory.java,v 1.3 2010/04/22 18:25:42 srevilak Exp $
 */

package edu.umb.bench.hedge;

import edu.umb.bench.framework.BaseTransactionFactory;
import edu.umb.bench.framework.ITransaction;
import edu.umb.bench.framework.INumberGenerator;
import edu.umb.bench.framework.GeneratorUnif;
import edu.umb.bench.framework.Probability;

/**
 * Hedge transaction factory.
 */
public class HedgeTransactionFactory
    extends BaseTransactionFactory<HedgeResult> {

    private INumberGenerator mGenerator;
    private HedgeTransaction mTransaction;
    private int nrows;
    private Card card;

    private static final double[] DEFAULT_SELECT_PROBABILITIES = {
        1.0, 0.0, 0.0, 0.0, 0.0, 0.0
    };

    private double selectProbabilities[];


    public void init() {
        mGenerator = new GeneratorUnif();
        mTransaction = new HedgeTransaction();

        commonSetup(mTransaction);

        nrows = (int) (getScale() * Constants.ROWS_PER_SCALE);

        String pdist = getConfig().getProperty(Constants.CONFIG_SELECT_DISTRIBUTION);

        if (pdist == null || pdist.length() == 0) {
            selectProbabilities = DEFAULT_SELECT_PROBABILITIES;
        }
        else {
            double tmp[] = Probability.parse(pdist);
            if (tmp.length == DEFAULT_SELECT_PROBABILITIES.length) {
                selectProbabilities = tmp;
            }
            else {
                getLogger().warn("using default probabilities, not "
                                 + Constants.CONFIG_SELECT_DISTRIBUTION
                                 + "="
                                 + pdist);
                selectProbabilities = DEFAULT_SELECT_PROBABILITIES;
            }
        }

        if (! Card.isInitialized()) {
            /* BROKEN HACK */
            Card.initCard(nrows, 
                          nrows / 2,
                          nrows / 4, 
                          nrows / 8,
                          nrows / 16,
                          nrows / 32);
        }



        card = Card.getInstance();
    }


    /**
     * Returns a configured transaction.
     */
    public ITransaction<HedgeResult> createTransaction() {
        /* Factories are per-thread, so we'll use the same
         * transaction profile, and just re-configure it each time 
         */

        int colPos = Probability.pickElement(selectProbabilities);
        String colname = card.getName(colPos);
        int cardinality = card.getCard(colPos);

        /* select according to column probability distribution.
         * Update according to primary key 
         *
         * FIXME: use hotspot */
        mTransaction.setParams(colname, 
                               colPos,
                               (int) (cardinality * mGenerator.getNumber()),
                               (int) (nrows * mGenerator.getNumber()));

        return mTransaction;
    }
}
