/*
 * $Id: Card.java,v 1.4 2010/06/26 01:10:20 srevilak Exp $
 */

package edu.umb.bench.sibench;

import static edu.umb.bench.sibench.Constants.COL_KSEQ;
import static edu.umb.bench.sibench.Constants.COL_KRANDSEQ;
import static edu.umb.bench.sibench.Constants.COL_K4;
import static edu.umb.bench.sibench.Constants.COL_K8;
import static edu.umb.bench.sibench.Constants.COL_K16;
import static edu.umb.bench.sibench.Constants.COL_K32;
import static edu.umb.bench.sibench.Constants.COL_K64;
import static edu.umb.bench.sibench.Constants.COL_K128;
import static edu.umb.bench.sibench.Constants.COL_K256;
import static edu.umb.bench.sibench.Constants.COL_K512;
import static edu.umb.bench.sibench.Constants.COL_K1024;
import static edu.umb.bench.sibench.Constants.COL_K2500;
import static edu.umb.bench.sibench.Constants.COL_K5K;
import static edu.umb.bench.sibench.Constants.COL_K10K;
import static edu.umb.bench.sibench.Constants.COL_K25K;
import static edu.umb.bench.sibench.Constants.COL_K50K;
import static edu.umb.bench.sibench.Constants.COL_K100K;
import static edu.umb.bench.sibench.Constants.COL_K250K;
import static edu.umb.bench.sibench.Constants.COL_K500K;

/**
 * Table of column cardinalities.  Given a column 
 * named K_N, we assume a domain of N different values.
 */
class Card {

    /**
     * Special cardinality value of a unique index.
     *
     * @see #getMaxValue
     */
    public static final int UNIQUE_INDEX = 0;

    /**
     * struct for pairing column names with column cardinalities
     */
    private static class Entry {
        String name;
        int value;
        public Entry(String n, int v) {
            this.name = n;
            this.value = v;
        }
        public String toString() {
            return this.name + " (" + this.value + ")";
        }
    }

    /**
     * Table of column cardinalities
     */
    private static Entry[] CARD_ENTRIES = new Entry[]{
        new Entry(COL_KSEQ,  UNIQUE_INDEX),
        new Entry(COL_KRANDSEQ, UNIQUE_INDEX),
        new Entry(COL_K4,    4),
        new Entry(COL_K8,    8),
        new Entry(COL_K16,   16),
        new Entry(COL_K32,   32),
        new Entry(COL_K64,   64),
        new Entry(COL_K128,  128),
        new Entry(COL_K256,  256),
        new Entry(COL_K512,  512),
        new Entry(COL_K1024, 1024),
        new Entry(COL_K2500, 2500),
        new Entry(COL_K5K,   5000),
        new Entry(COL_K10K,  10000),
        new Entry(COL_K25K,  25000),
        new Entry(COL_K50K,  50000),
        new Entry(COL_K100K, 100000),
        new Entry(COL_K250K, 250000),
        new Entry(COL_K500K, 500000)
    };
            

    /**
     * Returns the minimum value (i.e., 1) for a secondary index
     * column.
     */
    public static int getMinValue(String colname) {
        return 1;
    }


    /**
     * Returns the maximum value for a non-unique secondary index column.
     *
     * <p>
     * Returns {@link #UNIQUE_INDEX} if <tt>colname</tt> is a unique index.
     * (in which case, the caller is expected to know the number of rows
     * in the table).
     * 
     * @exception IllegalArgumentException if <tt>colname</tt> is not
     * a valid column name
     */
    public static int getMaxValue(String colname) {
        for (int iter = 0; iter < CARD_ENTRIES.length; iter++) {
            if (CARD_ENTRIES[iter].name.equals(colname)) {
                return CARD_ENTRIES[iter].value;
            }
        }
        throw new IllegalArgumentException("invalid column name: " + colname);
    }

}
