/*
 * $Id: Constants.java,v 1.14 2011/09/07 11:36:31 srevilak Exp $
 * $Source: /home/srevilak/Documents/schoolwork/phd/REPOSITORY/cvsroot/innobase-5.1.31-si/benchmark/src/edu/umb/bench/sibench/Constants.java,v $
 */

package edu.umb.bench.sibench;

/**
 * Constants used by sibench benchmark.
 *
 * @version 1.13 2011/09/07 11:35:05
 */
public class Constants {
    
    private Constants() {
        /* just constants */
    }

    /**
     * Rows per scale ({@value}).
     */
    public static final int ROWS_PER_SCALE = 1000000;

    
    /**
     * Property key holding number of rows.
     */
    public static final String KEY_NUM_ROWS = "num-rows";

    /**
     * Configuation key with a probability distribution
     * for choosing select columns
     */
    public static final String CONFIG_SELECT_DISTRIBUTION
        = "bench.select.distribution";


    /**
     * Configuration key that determines how often we select
     * rows from within a hotspot */
    public static final String CONFIG_HOTSPOT_PROBABILITY = 
        "config.hotspot.probability";


    /**
     * Configuration key that determines the number of
     * rows in the hotspot, per {@link #ROWS_PER_SCALE}
     */
    public static final String CONFIG_HOTSPOT_SIZE = 
        "config.hotspot.size";

    /**
     * Configuration key that specifies how many selects to do
     * ({@value}).  
     */
    public static final String CONFIG_SELECT_STATEMENTS =
        "config.select.statements";

    /**
     * For sibench2m, the number of selected rows in a krandseq
     * range scan ({@value}).
     */
    public static final String CONFIG_SELECT_RANGE = 
        "config.select.range";


    /**
     * Configuration key that specifies how many updates to do
     * ({@value}).  
     */
    public static final String CONFIG_UPDATE_STATEMENTS =
        "config.update.statements";

    /**
     * Configuration key that specifies the base duration of a
     * randomized delay between statements.  The duration is floating
     * point number, given is milliseconds. ({@value}).
     * 
     * <p> Given a base duration of $N$, we'll sleep $N$ milliseconds
     * +/- 50% after each SQL statement, with the exception of the
     * last.  (There is no delay between the last statement and
     * commit.)  
     */
    public static final String CONFIG_RANDOM_SLEEP =
        "config.random.sleep";

    /**
     * Configuration key that specified how much the hotspot shifts
     * after each transaction ({@value}).  sibench2m only 
     */
    public static final String CONFIG_HOTSPOT_SHIFT = 
        "config.hotspot.shift";


    /**
     * Default size of hotspot, as a percentage of table size.
     */
    public static final double DEFAULT_HOTSPOT_SIZE = 
        500.0 / (1.0 * ROWS_PER_SCALE);



    /**
     * Default hotspot probability.
     */
    public static final double DEFAULT_HOTSPOT_PROBABILITY = 0.9;

    /**
     * Result key for the number of times that kseq was used in select.
     */
    public static final String SELECT_KSEQ_COUNT = "select-kseq-count";

    /**
     * Result key for number of times that krandseq was used in select
     */
    public static final String SELECT_KRANDSEQ_COUNT = "select-krandseq-count";

    /**
     * Result key for number of times that k1 was used in select
     */
    public static final String SELECT_K4_COUNT = "select-k0004-count";

    /**
     * Result key for number of times that k8 was used in select
     */
    public static final String SELECT_K8_COUNT = "select-k0008-count";
        
    /**
     * Result key for number of times that k16 was used in select
     */
    public static final String SELECT_K16_COUNT = "select-k0016-count";
        

    /**
     * Result key for number of times that k32 was used in select
     */
    public static final String SELECT_K32_COUNT = "select-k0032-count";
        
    /**
     * Result key for number of times that k64 was used in select
     */
    public static final String SELECT_K64_COUNT = "select-k0064-count";

    /**
     * Result key for number of times that k128 was used in select
     */
    public static final String SELECT_K128_COUNT = "select-k0128-count";

    /**
     * Result key for number of times that k256 was used in select
     */
    public static final String SELECT_K256_COUNT = "select-k0256-count";

    /**
     * Result key for number of times that k512 was used in select
     */
    public static final String SELECT_K512_COUNT = "select-k0512-count";

    /**
     * Result key for number of times that k1024 was used in select
     */
    public static final String SELECT_K1024_COUNT = "select-k1024-count";

    /**
     * Result key for number of times that k2500 was used in select
     */
    public static final String SELECT_K2500_COUNT = "select-k2500-count";

    /**
     * Result key for number of times that k5k was used in select
     */
    public static final String SELECT_K5K_COUNT = "select-k5k-count";

    /**
     * Result key for number of times that k10k was used in select
     */
    public static final String SELECT_K10K_COUNT = "select-k10k-count";

    /**
     * Result key for number of times that k25k was used in select
     */
    public static final String SELECT_K25K_COUNT = "select-k25k-count";

    /**
     * Result key for number of times that k50k was used in select
     */
    public static final String SELECT_K50K_COUNT = "select-k50k-count";

    /**
     * Result key for number of times that k100k was used in select
     */
    public static final String SELECT_K100K_COUNT = "select-k100k-count";

    /**
     * Result key for number of times that k250k was used in select
     */
    public static final String SELECT_K250K_COUNT = "select-k250k-count";

    /**
     * Result key for number of times that k500k was used in select
     */
    public static final String SELECT_K500K_COUNT = "select-k500k-count";


    /**
     * Result key containing the total number of milliseconds spent
     * executing SQL statements.  This total includes transactions
     * that successfully commit, but not transactions that abort.
     */
    public static final String SUM_EXEC_DURATION_MS = "sum-exec-duration-ms";

    /**
     * Result key containing the total number of milliseconds spent
     * executing commit statements.  This total includes transactions
     * that successfully commit, but not transactions that abort.
     */
    public static final String SUM_COMMIT_DURATION_MS = "sum-commit-duration-ms";

    /**
     * Array of SELECT_.._COUNT result keys.  These (must) appear
     * in the same order as {@link #COL_NAMES}.
     */
    public static final String SELECT_COUNT_KEYS[] = {
        SELECT_KSEQ_COUNT,
        SELECT_KRANDSEQ_COUNT,
        SELECT_K4_COUNT,
        SELECT_K8_COUNT,
        SELECT_K16_COUNT,
        SELECT_K32_COUNT,
        SELECT_K64_COUNT,
        SELECT_K128_COUNT,
        SELECT_K256_COUNT,
        SELECT_K512_COUNT,
        SELECT_K1024_COUNT,
        SELECT_K2500_COUNT,
        SELECT_K5K_COUNT,
        SELECT_K10K_COUNT,
        SELECT_K25K_COUNT,
        SELECT_K50K_COUNT,
        SELECT_K100K_COUNT,
        SELECT_K250K_COUNT,
        SELECT_K500K_COUNT
    };


    public static final String COL_KSEQ = "kseq";
    public static final String COL_KRANDSEQ = "krandseq";
    public static final String COL_K4 = "k4";
    public static final String COL_K8 = "k8";
    public static final String COL_K16 = "k16";
    public static final String COL_K32 = "k32";
    public static final String COL_K64 = "k64";
    public static final String COL_K128 = "k128";
    public static final String COL_K256 = "k256";
    public static final String COL_K512 = "k512";
    public static final String COL_K1024 = "k1024";
    public static final String COL_K2500 = "k2500";
    public static final String COL_K5K = "k5k";
    public static final String COL_K10K = "k10k";
    public static final String COL_K25K = "k25k";
    public static final String COL_K50K = "k50k";
    public static final String COL_K100K = "k100k";
    public static final String COL_K250K = "k250k";
    public static final String COL_K500K = "k500k";

    public static final String COL_UPDATE = COL_KRANDSEQ;

    /** 
     * Array of k column names, in ordinal position order.
     * The ordering should match {@link #SELECT_COUNT_KEYS}.
     */
    public static final String COL_NAMES[] = {
        COL_KSEQ,
        COL_KRANDSEQ,
        COL_K4,
        COL_K8,
        COL_K16,
        COL_K32,
        COL_K64,
        COL_K128,
        COL_K256,
        COL_K512,
        COL_K1024,
        COL_K2500,
        COL_K5K,
        COL_K10K, 
        COL_K25K,
        COL_K50K,
        COL_K100K,
        COL_K250K,
        COL_K500K
    };

    static {
        if (COL_NAMES.length != SELECT_COUNT_KEYS.length) {
            throw new IllegalStateException
                ("COL_NAMES and SELECT_COUNT_KEYS must have the "
                 + "same number of elements.  Please fix.");
        }
    }

}
