/*
 * $Id: SIBench1Transaction.java,v 1.18 2011/06/03 01:16:06 srevilak Exp $
 */

package edu.umb.bench.sibench;

import edu.umb.bench.framework.BaseTransaction;
import edu.umb.bench.framework.Util;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;


/**
 * Transaction for SIBench1.
 *
 * @version $Revision: 1.18 $ $Date: 2011/06/03 01:16:06 $
 */
public class SIBench1Transaction extends BaseTransaction<TxResult> {
    
    private TxResult mResult;

    private String mSelectColumn;
    private int mSelectKeys[];
    private int mUpdateKeys[];
    private double mRandomSleep;

    private long statement_start = 0;
    private long commit_start = 0;
    private long commit_end = 0;


    /**
     * Pseudo #ifdef.  If enabled, record the amount of time
     * spent executing SQL statements, and the amount of time
     * spent executing commit statements.
     */
    private static final boolean MICRO_TIMING = true;

    public SIBench1Transaction() {
        super();
        mResult = new TxResult();
    }

    public void setParams(String selectColumn, 
                          int selectKeys[],
                          int updateKeys[],
                          double randomSleep) {
        mSelectColumn = selectColumn;
        mSelectKeys = selectKeys;
        mUpdateKeys = updateKeys;
        mRandomSleep = randomSleep;
    }


    public void execute(boolean warmup) throws SQLException {
        PreparedStatement p1 = null;
        PreparedStatement p2 = null;
        Connection conn = getConnection();
        ResultSet rs = null;

        mResult.reset();
        mResult.setColumn(mSelectColumn);

        try {
            
            statement_start = 0;
            commit_start = 0;
            commit_end = 0;
            
            if (MICRO_TIMING) {
                statement_start = System.currentTimeMillis();
            }

            int delta = 0;
            String sql = "select avg(kval) v from bench where " 
                + mSelectColumn + " =  ?";

            p1 = conn.prepareStatement(sql);

            for (int iter = 0; iter < mSelectKeys.length; iter++) {

                p1.setInt(1, mSelectKeys[iter]);
                p1.executeQuery();
                rs = p1.getResultSet();

                delta += readRs(rs);
                
                /* random sleep if there is more work to do */
                if (mUpdateKeys.length > 0
                    || (iter < mSelectKeys.length - 1)) {
                    Util.randomSleepQuiet(mRandomSleep);
                }
            }            
            p1.close();

            /* Delta is +/- v for v in [1,10] */
            delta = (delta % 10000) + 1000;
            if (Math.random() < 0.50) {
                delta = delta * -1;
            }

            p2 = conn.prepareStatement
                ("update bench set kval = kval + (0.001 * ?) "
                 + "where " + Constants.COL_UPDATE + " = ?");

            for (int iter = 0; iter < mUpdateKeys.length; iter++) {
                p2.setInt(1, delta);
                p2.setInt(2, mUpdateKeys[iter]);

                int rv = p2.executeUpdate();
                assert(rv == 1);

                if (iter < (mUpdateKeys.length - 1)) {
                    /* no delay after last statement */
                    Util.randomSleepQuiet(mRandomSleep);
                }
            }
        }

        finally {

            if (p1 != null) {
                p1.close();
            }

            if (p2 != null) {
                p2.close();
            }
        }

    }


    public void commit() throws SQLException {

        if (MICRO_TIMING) {
            commit_start = System.currentTimeMillis();
        }

        super.commit();

        if (MICRO_TIMING) {
            commit_end = System.currentTimeMillis();
            mResult.setExecDuration(commit_start - statement_start);
            mResult.setCommitDuration(commit_end - commit_start);
        }
    }
    

    private int readRs(ResultSet rs) throws SQLException {
        assert rs != null;
        long sum = 0;

        try {
            while (rs.next()) {
                sum += rs.getLong("v");
            }
        }

        finally {
            rs.close();
        }

        return (int) (sum & 0x7fffffffL);
    }


    public TxResult getResults() {
        return mResult;
    }

}
