/*
 * $Id: SIBenchMTransaction.java,v 1.2 2010/07/25 03:17:20 srevilak Exp $
 */

package edu.umb.bench.sibench;

import edu.umb.bench.framework.BaseTransaction;
import edu.umb.bench.framework.Util;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.sql.Connection;
import java.sql.ResultSet;

/**
 * Transaction for sibench-M.
 *
 * @version $Revision: 1.2 $ $Date: 2010/07/25 03:17:20 $
 */
public class SIBenchMTransaction extends BaseTransaction<TxResult> {
    
    private static final String PRED_COLUMN = Constants.COL_KSEQ;

    private TxResult mResult;

    private int mSelectLow;
    private int mSelectHigh;
    private int mUpdateKeys[];
    private double mRandomSleep;

    public SIBenchMTransaction() {
        super();
        mResult = new TxResult();
    }

    public void setParams(int selectLow, 
                          int selectHigh,
                          int updateKeys[],
                          double randomSleep) {
        mSelectLow = selectLow;
        mSelectHigh = selectHigh;
        mUpdateKeys = updateKeys;
        mRandomSleep = randomSleep;
    }


    private static final String UPDATE_SQL = ""
        + "update bench set kval = kval + (0.001 * ?) "
        + "where "
        + PRED_COLUMN
        + " = ?";

    public void execute(boolean warmup) throws SQLException {
        Statement p1 = null;
        PreparedStatement p2 = null;
        Connection conn = getConnection();
        ResultSet rs = null;

        mResult.reset();
        mResult.setColumn(PRED_COLUMN);

        try {
            
            int delta = 0;
            String sql1 = new StringBuilder(128) 
                .append("select avg(kval) v from bench where ")
                .append(PRED_COLUMN)
                .append(" between ")
                .append(mSelectLow)
                .append(" and ")
                .append(mSelectHigh)
                .toString();

            p1 = conn.createStatement();
            p1.executeQuery(sql1);
            rs = p1.getResultSet();

            delta += readRs(rs);
            
            /* random sleep if there is more work to do */
            if (mUpdateKeys.length > 0) {
                Util.randomSleepQuiet(mRandomSleep);
            }
            
            p2 = conn.prepareStatement(UPDATE_SQL);
            
            for (int iter = 0; iter < mUpdateKeys.length; iter++) {
                
                p2.setInt(1, 1000 + (delta % 1000));
                p2.setInt(2, mUpdateKeys[iter]);
                p2.executeUpdate();
                
                if (iter < (mUpdateKeys.length - 1)) {
                    /* no delay after last statement */
                    Util.randomSleepQuiet(mRandomSleep);
                }
            }

            conn.commit();
        }

        finally {

            if (p1 != null) {
                p1.close();
            }

            if (p2 != null) {
                p2.close();
            }
        }

    }


    private int readRs(ResultSet rs) throws SQLException {
        assert rs != null;
        long sum = 0;

        try {
            while (rs.next()) {
                sum += rs.getLong("v");
            }
        }

        finally {
            rs.close();
        }

        return (int) (sum & 0x7fffffffL);
    }


    public TxResult getResults() {
        return mResult;
    }

}
