/*
 * $Id: SIBenchSetup.java,v 1.15 2011/03/23 01:54:40 srevilak Exp $
 */

package edu.umb.bench.sibench;

import java.sql.SQLException;
import edu.umb.bench.framework.BaseSetup;
import edu.umb.bench.framework.ILogger;
import edu.umb.bench.framework.Util;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.sql.ResultSet;
import edu.umb.bench.framework.INumberGenerator;
import edu.umb.bench.framework.GeneratorUnif;
import edu.umb.bench.framework.Probability;
import java.util.Properties;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;

/**
 * Setup class for sibench benchmark.
 *
 * @version $Revision: 1.15 $ $Date: 2011/03/23 01:54:40 $
 */
public class SIBenchSetup extends BaseSetup {


    public static final String DROP_SQL = 
        "drop table if exists bench";

    public static final String CREATE_SQL = ""
        + "create table bench ("
        + " kseq integer not null primary key, "
        + " krandseq integer not null, "
        + " k4 integer not null, "
        + " k8 integer not null, "
        + " k16 integer not null, "
        + " k32 integer not null, "
        + " k64 integer not null, "
        + " k128 integer not null, "
        + " k256 integer not null, "
        + " k512 integer not null, "
        + " k1024 integer not null, "
        + " k2500 integer not null, "
        + " k5k   integer not null, "
        + " k10k  integer not null, "
        + " k25k  integer not null, "
        + " k50k  integer not null, "
        + " k100k integer not null, "
        + " k250k integer not null, "
        + " k500k integer not null, "
        + " kval  integer not null, "
        + " pad varchar(20), "
        + "unique index krandseq_idx(krandseq), "
        + "index k4_idx(k4), "
        + "index k8_idx(k8), "
        + "index k16_idx(k16), "
        + "index k32_idx(k32), "
        + "index k64_idx(k64), "
        + "index k128_idx(k128), "
        + "index k256_idx(k256), "
        + "index k512_idx(k512), "
        + "index k1024_idx(k1024), "
        + "index k2500_idx(k2500), "
        + "index k5k_idx(k5k), "
        + "index k10k_idx(k10k), "
        + "index k25k_idx(k25k), "
        + "index k50k_idx(k50k), "
        + "index k100k_idx(k100k), "
        + "index k250k_idx(k250k), "
        + "index k500k_idx(k500k) "
        + ") ENGINE=InnoDB";
    
    private INumberGenerator gen;


    public SIBenchSetup() {
        super();
        gen = new GeneratorUnif();
    }


    public void createSchema() throws SQLException {

        if (tableOK()) {
            return;
        }

        Connection conn = getConnection();
        ILogger log = getLogger();
        Statement st = null;
        log.info("creating bench table");

        try {
            st = conn.createStatement();
            st.execute(DROP_SQL);
            st.execute(CREATE_SQL);
            conn.commit();
        }
        finally {
            if (st != null) {
                st.close();
            }
        }
    }


    public void fillTables() throws SQLException {

        if (tableOK()) {
            getLogger().info("table OK as-is");
            return;
        }
        
        Connection conn = getConnection();
        ILogger log = getLogger();
        PreparedStatement ps = null;

        int nrows = getNRows();

        log.info("filling bench (" + nrows + " rows)");
        List<Integer> randseq = makeRandomSequence(nrows);

        try {
            ps = conn.prepareStatement
                ("insert into bench "
                 + "(kseq, krandseq,"
                 + "k4,  k8,   k16,  k32, "
                 + "k64, k128, k256, k512,"
                 + "k1024, k2500, k5k, k10k, "
                 + "k25k, k50k, k100k, "
                 + "k250k, k500k, "
                 + "kval, pad) " 
                 + "values "
                 + "(?,?,"
                 + " ?,?,?,?,"
                 + " ?,?,?,?,"
                 + " ?,?,?,?,"
                 + " ?,?,?,"
                 + " ?,?,"
                 + " ?,?)");

            for (int iter = 1; iter <= nrows; iter++) {
                
                int kseq = iter;
                int krandseq = randseq.get(iter - 1).intValue();
                int k4   = 1 + (int) (Card.getMaxValue("k4") * gen.getNumber());
                int k8   = 1 + (int) (Card.getMaxValue("k8") * gen.getNumber());
                int k16  = 1 + (int) (Card.getMaxValue("k16") * gen.getNumber());
                int k32  = 1 + (int) (Card.getMaxValue("k32") * gen.getNumber());
                int k64  = 1 + (int) (Card.getMaxValue("k64") * gen.getNumber());
                int k128 = 1 + (int) (Card.getMaxValue("k128") * gen.getNumber());
                int k256 = 1 + (int) (Card.getMaxValue("k256") * gen.getNumber());
                int k512 = 1 + (int) (Card.getMaxValue("k512") * gen.getNumber());
                int k1024 = 1 + (int) (Card.getMaxValue("k1024") * gen.getNumber());
                int k2500 = 1 + (int) (Card.getMaxValue("k2500") * gen.getNumber());
                int k5k   = 1 + (int) (Card.getMaxValue("k5k") * gen.getNumber());
                int k10k  = 1 + (int) (Card.getMaxValue("k10k") * gen.getNumber());
                int k25k  = 1 + (int) (Card.getMaxValue("k25k") * gen.getNumber());
                int k50k  = 1 + (int) (Card.getMaxValue("k50k") * gen.getNumber());
                int k100k = 1 + (int) (Card.getMaxValue("k100k") * gen.getNumber());
                int k250k = 1 + (int) (Card.getMaxValue("k250k") * gen.getNumber());
                int k500k = 1 + (int) (Card.getMaxValue("k500k") * gen.getNumber());

                int val = ((int) (gen.getNumber() * 90000)) + 10000;
                String pad = makePad();
                
                int index = 1;
                ps.setInt(index++, kseq);
                ps.setInt(index++, krandseq);
                ps.setInt(index++, k4);
                ps.setInt(index++, k8);
                ps.setInt(index++, k16);
                ps.setInt(index++, k32);
                ps.setInt(index++, k64);
                ps.setInt(index++, k128);
                ps.setInt(index++, k256);
                ps.setInt(index++, k512);
                ps.setInt(index++, k1024);
                ps.setInt(index++, k2500);
                ps.setInt(index++, k5k);
                ps.setInt(index++, k10k);
                ps.setInt(index++, k25k);
                ps.setInt(index++, k50k);
                ps.setInt(index++, k100k);
                ps.setInt(index++, k250k);
                ps.setInt(index++, k500k);
                ps.setInt(index++, val);
                ps.setString(index++, pad);

                ps.execute();

                if (iter > 0 && iter % 100000 == 0) {
                    getLogger().info(iter + " rows inserted");
                }
            }

            conn.commit();
        }

        finally {
            if (ps != null) {
                ps.close();
            }
        }
    }



    public void warmup() throws SQLException {
        Connection conn = getConnection();
        Statement st = null;
        ResultSet rs = null;

        try {
            st = conn.createStatement();

            /* trim large values, to avoid integer overflow */
            st.execute("update bench set kval = (10000 + (rand()*90000)) "
                       + "where kval > 100000000 or "
                       + "kval < -100000000");

            conn.commit();

            /* bring in main table data */
            String sql = "select avg(kval) from bench";
            getLogger().info(sql);
            st.execute(sql);
            rs = st.getResultSet();
            resultSetSlurp(rs);
            rs.close();
            conn.commit();

            /* for each column that we'll use in the test,
             * bring that column's index into buffer. */
            String pstring = 
                getConfig().getProperty(Constants.CONFIG_SELECT_DISTRIBUTION);

            double pdist[] = 
                SIBench1TransactionFactory.DEFAULT_SELECT_PROBABILITIES;

            if (pstring != null && pstring.length() > 0) {
                pdist = Probability.parse(pstring);
            }

            for (int iter = 1; 
                 iter < pdist.length && iter < Constants.COL_NAMES.length;
                 iter++) {

                /* always warmup krandseq, which is iter == 1 */
                if (iter != 1 && pdist[iter] == 0.0) {
                    continue;
                }

                /* NOTE: relies on distribution values having
                 * the same ordinal positions as columns */
                String column = Constants.COL_NAMES[iter];
                warmupColumn(column);
            }

        }
        catch (SQLException e) {

            try {
                conn.rollback();
            }
            catch (Exception e2) {
                e2.printStackTrace();
            }

            if (st != null) {
                st.close();
            }
            if (rs != null) {
                rs.close();
            }

            throw new RuntimeException("error during warmup", e);
        }
    }


    private void warmupColumn(String column) throws SQLException {
        /* First, the number of distinct column values.  Count
         * distinct is really overkill, but hey, we're trying to warm
         * up the table */
        String sql = "select count(distinct " + column + ") c from bench";
        Connection conn = getConnection();

        getLogger().info(sql);

        PreparedStatement ps = conn.prepareStatement(sql);
        ps.execute();

        ResultSet rs = ps.getResultSet();

        int n_distinct = 0;
        while (rs.next()) {
            n_distinct = rs.getInt("c");
        }
        rs.close();
        ps.close();
        conn.commit();

        int skip = n_distinct / 250;
        if (skip < 1) {
            skip = 1;
        }

        /* now, add one to each kval; every skip-th row */
        sql = "update bench set kval = kval + 1 where " + column + " = ?";
        getLogger().info(sql);
        ps = conn.prepareStatement(sql);
        for (int iter = 1; iter <= n_distinct; iter += skip) {
            ps.setInt(1, iter);
            ps.execute();
        }
        conn.commit();
        ps.close();

        Util.randomSleepQuiet(250.0);

        /* now, take one away from each kval */
        sql = "update bench set kval = kval - 1 where " + column + " = ?";
        getLogger().info(sql);
        ps = conn.prepareStatement(sql);
        for (int iter = 1; iter <= n_distinct; iter += skip) {
            ps.setInt(1, iter);
            ps.execute();
        }
        conn.commit();
        ps.close();
        
        Util.randomSleepQuiet(250.0);

        /* now, select between values */
        sql = "select avg(kval) from bench where " + column + " = ?";
        getLogger().info(sql);
        ps = conn.prepareStatement(sql);
        for (int iter = (skip/2); iter <= n_distinct; iter += skip) {
            ps.setInt(1, iter);
            ps.execute();
            rs = ps.getResultSet();
            resultSetSlurp(rs);
            rs.close();
        }
        conn.commit();
        ps.close();

    }


    private int resultSetSlurp(ResultSet rs) throws SQLException {
        int count = 0;

        while (rs.next()) {
            count++;
        }

        return count;
    }


    private List<Integer> makeRandomSequence(int nrows) {
        List<Integer> list = new ArrayList<Integer>(nrows);
        for (int iter = 1; iter <= nrows; iter++) {
            list.add(Integer.valueOf(iter));
        }

        Collections.shuffle(list);
        Collections.reverse(list);
        Collections.shuffle(list);

        return list;
    }


    private String makePad() {
        final int padlen = 20;

        int nchars = 'z' - 'a';

        StringBuilder sb = new StringBuilder(padlen);
        for (int iter = 0; iter < padlen; iter++) {
            sb.append((char) ((gen.getNumber() * nchars) + 'a'));
        }
        return sb.toString();
    }


    private boolean tableOK() throws SQLException {
        Connection conn = getConnection();
        Statement st = null; 
        ResultSet rs = null;
        int max_seq = 0;
        int nrows = getNRows();

        /* store nrows for later use.  This will also put
         * nrows into the test results file */
        Properties config = getConfig();
        config.put(Constants.KEY_NUM_ROWS, Integer.toString(nrows));

        try {
            st = conn.createStatement();
            st.execute("select max(kseq) from bench");
            rs = st.getResultSet();

            while (rs.next()) {
                max_seq = rs.getInt(1);
            }
            conn.commit();

            if (nrows == max_seq) {
                return true;
            }
        }

        catch (SQLException e) {

            String msg = e.getMessage();

            if (msg.indexOf("doesn't exist") >= 0) {
                getLogger().info(msg);
                return false;
            }
 
            else {
                throw e;
            }
        }

        finally {
            if (rs != null) {
                rs.close();
            }

            if (st != null) {
                st.close();
            }
        }
        return false;
    }


    private int getNRows() {
        return (int) (getScale() * Constants.ROWS_PER_SCALE);
    }
}
