/*
 * $Id: SIBenchWorker.java,v 1.4 2011/02/23 16:27:43 srevilak Exp $
 */

package edu.umb.bench.sibench;

import edu.umb.bench.framework.BaseWorker;


import java.util.Map;
import java.util.HashMap;

import static edu.umb.bench.sibench.Constants.COL_KSEQ;
import static edu.umb.bench.sibench.Constants.COL_KRANDSEQ;
import static edu.umb.bench.sibench.Constants.COL_K4;
import static edu.umb.bench.sibench.Constants.COL_K8;
import static edu.umb.bench.sibench.Constants.COL_K16;
import static edu.umb.bench.sibench.Constants.COL_K32;
import static edu.umb.bench.sibench.Constants.COL_K64;
import static edu.umb.bench.sibench.Constants.COL_K128;
import static edu.umb.bench.sibench.Constants.COL_K256;
import static edu.umb.bench.sibench.Constants.COL_K512;
import static edu.umb.bench.sibench.Constants.COL_K1024;

import static edu.umb.bench.sibench.Constants.SELECT_KRANDSEQ_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_KSEQ_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_K4_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_K8_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_K16_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_K32_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_K64_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_K128_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_K256_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_K512_COUNT;
import static edu.umb.bench.sibench.Constants.SELECT_K1024_COUNT;


/**
 * SIBench Worker thread.
 *
 * @version $Revision: 1.4 $ $Date: 2011/02/23 16:27:43 $
 */
public class SIBenchWorker extends BaseWorker<TxResult> {

    private static class IntHolder {
        public int value;
    }
    
    Map<String,IntHolder> mSelectColumns;
    private long execDuration = 0;
    private long commitDuration = 0;

    public SIBenchWorker() {
        super();
        mSelectColumns = new HashMap<String,IntHolder>();
    }

    /**
     * Adds the results for a single transaction.  At present,
     * this is a no-op.
     */
    public void addResult(TxResult result) {
        String column = result.getColumn();
        IntHolder val = mSelectColumns.get(column);
        if (val != null) {
            val.value++;
        }
        else {
            val = new IntHolder();
            val.value++;
            mSelectColumns.put(column, val);
        }

        long exec = result.getExecDuration();
        long commit = result.getCommitDuration();
        if (commit >= 0) {
            execDuration += exec;
            commitDuration += commit;
        }
    }

    
    public Map<String,Object> getResults() {

        Map<String,Object> results = new HashMap<String,Object>();

        for (int iter = 0; iter < Constants.COL_NAMES.length; iter++) {

            int count = getSelectCount(Constants.COL_NAMES[iter]);

            results.put(Constants.SELECT_COUNT_KEYS[iter],
                        Integer.valueOf(count));
        }

        results.put(Constants.SUM_EXEC_DURATION_MS, 
                    Long.valueOf(execDuration));
        results.put(Constants.SUM_COMMIT_DURATION_MS,
                    Long.valueOf(commitDuration));

        return results;
    }


    private int getSelectCount(String colname) {
        IntHolder val = mSelectColumns.get(colname);
        if (val == null) {
            return 0;
        }
        return val.value;
    }
}
