/*
 * $Id: SimpleAuditor.java,v 1.2 2010/01/31 19:25:04 srevilak Exp $
 */

package edu.umb.bench.simple;

import java.sql.SQLException;
import java.util.Map;
import java.util.HashMap;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.Connection;

import edu.umb.bench.framework.BaseAuditor;


/**
 * Auditor for simple benchmark.  Each audit method simply counts
 * the number of rows in the table.
 */
public class SimpleAuditor extends BaseAuditor {

    private Map<String,Object> mData;

    public SimpleAuditor() {
        mData = new HashMap<String,Object>();
    }

    public void postSetup() throws SQLException {
        getLogger().info("post setup audit");

        commonAudit(Constants.AUDIT_COUNT_BEFORE);
    }

    public void postTest() throws SQLException {
        getLogger().info("post test audit");
        
        commonAudit(Constants.AUDIT_COUNT_AFTER);
    }


    private void commonAudit(String key) throws SQLException {
        int cnt = countRows();
        mData.put(key, Integer.valueOf(cnt));
    }

    private int countRows() throws SQLException {
        Statement st = null;
        ResultSet rs = null;
        Connection conn = getConnection();
        int cnt = 0;

        try {

            st = conn.createStatement();
            st.execute("select count(*) cnt from simple");
            rs = st.getResultSet();

            while (rs.next()) {
                cnt = rs.getInt("cnt");
            }
            rs.close();
            conn.commit();
        }

        finally {
            if (st != null) {
                st.close();
            }
        }

        return cnt;
    }

    public Map<String,Object> getResults() {
        return mData;
    }
    
}
