/*
 * $Id: SimpleSetup.java,v 1.6 2010/04/22 23:16:33 srevilak Exp $
 */

package edu.umb.bench.simple;

import java.sql.SQLException;
import edu.umb.bench.framework.BaseSetup;
import edu.umb.bench.framework.ILogger;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.Statement;
import edu.umb.bench.framework.INumberGenerator;
import edu.umb.bench.framework.GeneratorUnif;

/**
 * Setup class for simple benchmark.
 *
 * @version $Revision: 1.6 $ $Date: 2010/04/22 23:16:33 $
 */
public class SimpleSetup extends BaseSetup {


    public static final String DROP_SQL = 
        "drop table if exists simple";

    public static final String DROP_PROC = 
        "drop procedure if exists simple_select";

    public static final String CREATE_SQL = ""
        + "create table simple "
        + "(id integer not null primary key, "
        + " value integer not null) "
        + "ENGINE=InnoDB";
    
    /* we have a variant that uses stored procedures instead of
     * prepared statements. */
    public static final String CREATE_PROC = ""
        + "create procedure simple_select "
        + "(in a integer, in b integer, out r integer)\n"
        + "begin\n"
        + "declare x integer;\n"
        + "declare y integer;\n"
        + "set x = (select sum(value) from simple where id = a);\n"
        + "set y = (select sum(value) from simple where id = b);\n"
        + "set r = ifnull(x, 0) + ifnull(y, 0);\n"
        + "commit;\n"
        + "end;\n"
        ;

    public void createSchema() throws SQLException {

        Connection conn = getConnection();
        ILogger log = getLogger();
        Statement st = null;
        log.info("creating schema");

        try {
            st = conn.createStatement();
            st.execute(DROP_SQL);
            // st.execute(DROP_PROC);
            st.execute(CREATE_SQL);
            // st.execute(CREATE_PROC);
            conn.commit();
        }
        finally {
            if (st != null) {
                st.close();
            }
        }

    }


    public void fillTables() throws SQLException {
        
        Connection conn = getConnection();
        ILogger log = getLogger();
        PreparedStatement ps = null;

        int nrows = (int) (getScale() * Constants.ROWS_PER_SCALE);
        INumberGenerator gen = new GeneratorUnif();

        log.info("filling table");
        try {
            ps = conn.prepareStatement("insert into simple (id, value) "
                                       + "values (?, ?)");
            for (int iter = 1; iter <= nrows; iter++) {
                ps.setInt(1, iter);
                ps.setInt(2, (int) (gen.getNumber() * 1000));
                ps.execute();
            }
            conn.commit();
        }

        finally {
            if (ps != null) {
                ps.close();
            }
        }
    }


}
