/*
 * $Id: ConnectionManagerTest.java,v 1.1 2010/02/03 15:41:16 srevilak Exp $
 */

package edu.umb.bench.framework;

import org.junit.Test;
import org.junit.Before;
import org.junit.After;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import java.util.Properties;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;

public class ConnectionManagerTest {

    private Properties connectionProperties;
    private ConnectionManager cm;


    @Before public void setup() {
        connectionProperties = TestUtil.getTestDBProperties();
        cm = new ConnectionManager(connectionProperties);
    }

    @After public void teardown() {
        if (cm != null) {
            cm.shutdown();
        }
    }

    public static void main(String argv[]) {
        TestUtil.runClass(ConnectionManagerTest.class);
    }


    /*==================================================================
     */


    /**
     * Connect to a database, issue a simple query, disconnect
     */
    @Test public void testSimpleConnect() throws SQLException {
        Connection conn = cm.getConnection();
        assertNotNull("getConnection", conn);

        Statement st = conn.createStatement();
        ResultSet rs = st.executeQuery("select 1 + 1 as x");
        
        int rv = 0;
        while (rs.next()) {
            rv = rs.getInt(1);
        }
        
        rs.close();
        st.close();

        assertEquals("select 1 + 1", 2, rv);
    }


    /**
     * Open several connections to the database,
     * verify that they are open.
     * Release the connections, verify that they are closed
     */
    @Test public void testCreateCloseSeveral() throws SQLException {
        Connection cxn[] = new Connection[10];

        for (int iter = 0; iter < cxn.length; iter++) {
            cxn[iter] = cm.getConnection();
        }

        for (int iter = 0; iter < cxn.length; iter++) {
            assertFalse("is open #" + iter, cxn[iter].isClosed());
        }

        cm.shutdown();

        for (int iter = 0; iter < cxn.length; iter++) {
            assertTrue("is closed #" + iter, cxn[iter].isClosed());
        }

    }
}
