/*
 * $Id: GenericHotspotTest.java,v 1.1 2010/04/26 00:38:52 srevilak Exp $
 */

package edu.umb.bench.framework;

import org.junit.Test;
import org.junit.Before;
import org.junit.After;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.junit.Assert.assertNotNull;
import java.util.Arrays;

public class GenericHotspotTest {

    private static boolean dump;

    private static final double EPS = 0.00001;

    @Before public void setup() {
        dump = (System.getProperty("DEBUG") != null);
        dump = true;
    }



    @After public void teardown() {

    }



    public static void main(String argv[]) {
        TestUtil.runClass(GenericHotspotTest.class);
    }


    @Test public void testBounds() {
        int counts[] = new int[10];
        IHotspot h = new GenericHotspot(10, 0, 0.0);
        
        /* for ten rows, we should choose values [1, 10] */
        for (int iter = 0; iter < 1000; iter++) {
            int value = h.getValue();
            assertTrue(">= 1: " + value, value >= 1);
            assertTrue("<= 10: " + value, value <= 10);
            counts[value - 1]++;
        }

        if (dump) {
            System.out.println(Arrays.toString(counts));
        }

        /* all values should be chosen */
        for (int iter = 0; iter < counts.length; iter++) {
            assertTrue("counts[" + iter + "]=" + counts[iter] + " > 0",
                       counts[iter] > 0);
        }
        
    }


    @Test public void testNinetyTen() {
        int counts[] = new int[10];
        IHotspot h = new GenericHotspot(10, 3, 0.9);
        
        for (int iter = 0; iter < 10000; iter++) {
            int value = h.getValue();
            assertTrue(">= 1: " + value, value >= 1);
            assertTrue("<= 10: " + value, value <= 10);
            counts[value - 1]++;
        }

        if (dump) {
            System.out.println(Arrays.toString(counts));
        }

        /* Check number 1.  All values should have been chosen */
        for (int iter = 0; iter < counts.length; iter++) {
            assertTrue("" + counts[iter] + " > 0", counts[iter] > 0);
        }

        /* check number 2.  Sum of hotspot counts should be
         * greater than the sum of non-hotspot counts */
        int hot = 0;
        int not_hot = 0;
        for (int iter = 0; iter < counts.length; iter++) {
            boolean isHot = (iter < 3);
            if (isHot) {
                hot += counts[iter];
            }
            else {
                not_hot += counts[iter];
            }
        }
        assertTrue("sum(hot) > sum(nothot)", hot > not_hot);
    }

}
