/*
 * $Id: ProbabilityTest.java,v 1.2 2010/03/08 17:34:02 srevilak Exp $
 */

package edu.umb.bench.framework;

import org.junit.Test;
import org.junit.Before;
import org.junit.After;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.junit.Assert.assertNotNull;

public class ProbabilityTest {

    private static final double EPS = 0.00001;

    @Before public void setup() {

    }



    @After public void teardown() {

    }



    public static void main(String argv[]) {
        TestUtil.runClass(ProbabilityTest.class);
    }



    /*==================================================================
     */

    @Test public void testScaleSimple() {
        double in[] = { 1.0, 2.0, 1.0 };
        double out[] = Probability.scale(in);
        
        assertEquals("out.length", in.length, out.length);
        assertEquals("out[0]", 0.25, out[0], EPS);
        assertEquals("out[1]", 0.50, out[1], EPS);
        assertEquals("out[2]", 0.25, out[2], EPS);
    }


    
    @Test public void testPick() {
        double in[] = Probability.scale(new double[]{ 1.0, 2.0, 1.0 });
        int results[] = new int[in.length];

        final int reps = 1000;

        for (int iter = 0; iter < reps; iter++) {
            int which = Probability.pickElement(in);
            assertTrue("pickElement", which >= 0 && which < results.length);
            results[which]++;
        }
        
        /* Make sure the output roughly follows the input 
         * probability distribution. */

        /* middle element should have been chosen most often */
        assertTrue("middle element largest",
                   results[1] > results[0] &&
                   results[1] > results[2]);

        /* first element chosen about 25% of the time */
        assertTrue("first element " + results[0],
                   results[0] >= (250 - 25) &&
                   results[0] <= (250 + 25));

        /* last element chosen about 25% of the time */
        assertTrue("last element " + results[2],
                   results[2] >= (250 - 25) &&
                   results[2] <= (250 + 25));

        /* middle element chosen about 50% of the time */
        assertTrue("middle element " + results[1],
                   results[1] >= (500 - 50) &&
                   results[1] <= (500 + 50));
        
    }



    @Test public void testParseEmptyString() {
        String in = "";
        try {
            Probability.parse(in);
            fail("no number format exception thrown for empty string");
        }
        catch (NumberFormatException e) {
            assertTrue("OK", true);
        }
    }


    
    @Test public void testParseComma() {
        String in = ",";
        try {
            Probability.parse(in);
            fail("no exception for parsing ','");
        }
        catch (NumberFormatException e) {
            assertTrue("OK", true);
        }
    }



    @Test public void testParseX() {
        String in = "x";
        try {
            Probability.parse(in);
            fail("no exception for input string 'x'");
        }
        catch (NumberFormatException e) {
            assertTrue("OK", true);
        }
    }



    @Test public void testParse1111() {
        String in = "1,1,1,1";
        double dist[] = Probability.parse(in);
        assertEquals("dist.length", 4, dist.length);
        for (int iter = 0; iter < dist.length; iter++) {
            assertEquals("dist[" + iter + "]", 0.25, dist[iter], EPS);
        }
    }



    @Test public void testParse0030() {
        String in = "0, 0, 3, 0";
        double dist[] = Probability.parse(in);
        assertEquals("dist.length", 4, dist.length);
    
        for (int iter = 0; iter < dist.length; iter++) {
            switch (iter) {
            case 2:
                assertEquals("dist[" + iter + "]", 1.0, dist[iter], EPS);
                break;
            default:
                assertEquals("dist[" + iter + "]", 0.0, dist[iter], EPS);
                break;
            }
        }
    }

}
