/* 
 * $Id: RandomHotspotTest.java,v 1.5 2010/05/22 21:32:44 srevilak Exp $
 */

package edu.umb.bench.framework;

import org.junit.Test;
import org.junit.Before;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import java.util.Set;
import java.util.Iterator;

public class RandomHotspotTest {

    private RandomHotspot h;
    
    private static final int TABLE_SIZE = 1000;
    private static final int HOT_ROWS = 100;
    private static final double PROBABILITY = 1.00;

    @Before public void setup() {
        h = new RandomHotspot(TABLE_SIZE, HOT_ROWS, PROBABILITY);
    }



    public static void main(String argv[]) {
        TestUtil.runClass(RandomHotspotTest.class);
    }


    @Test public void testHotspotGeneration() {
        Set<Integer> hot = h.getHotspot();
        
        assertEquals("length", HOT_ROWS, hot.size());

        Iterator<Integer> iter = hot.iterator();
        while (iter.hasNext()) {
            Integer value0 = iter.next();
            int value = value0.intValue();
            assertTrue("not too low", value >= 1);
            assertTrue("not too high", value <= TABLE_SIZE);
        }
    }


    /** 100% of rows are hot, so all should be found
     * in the hot array
     */
    @Test public void testHotspotSelection() {
        Set<Integer> hot = h.getHotspot();

        for (int iter = 0; iter < 100; iter++) {
            int value = h.getValue();
            assertTrue("found", hot.contains(Integer.valueOf(value)));
        }
    }
}
