/*
 * $Id: TestUtil.java,v 1.2 2010/02/03 15:43:12 srevilak Exp $
 */

package edu.umb.bench.framework;

import org.junit.runner.JUnitCore;
import static org.junit.Assert.fail;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.Properties;
import java.io.InputStream;

public class TestUtil {

    private static final String TEST_DB_PROPERTIES_PATH 
        = "edu/umb/bench/framework/test.db.properties";

    /**
     * Run tests in testclass.  This is a wrapper
     * around one of JUnit's command line test runners.
     */
    public static void runClass(Class<?> testclass) {
        JUnitCore.main(new String[] {testclass.getName()});
    }

    
    public static Properties getTestDBProperties() {
        try {
            return loadProperties(TEST_DB_PROPERTIES_PATH);
        }
        catch (IOException e) {
            fail("failed loading " + TEST_DB_PROPERTIES_PATH
                 + ": " + e.toString());
        }

        return null; /* not reached */
    }


    private static Properties loadProperties(String resourcePath) 
        throws IOException {

        ClassLoader loader = TestUtil.class.getClassLoader();
        InputStream in = loader.getResourceAsStream(resourcePath);

        if (in == null) {
            throw new FileNotFoundException("missing resource: " 
                                            + resourcePath);
        }

        Properties p = new Properties();
        p.load(in);

        in.close();

        return p;
    }

}
