/*
 * $Id: lock0hash.c,v 1.3 2009/11/27 18:25:00 srevilak Exp $ 
 *
 */

#define LOCK_MODULE_IMPLEMENTATION

#include "lock0hash.h"
#include "lock0lock.h"

lock_hash_table_t * lock_hash_create(ulint n) {
	lock_hash_cell_t * array;
	ulint prime;
	lock_hash_table_t * table;
	ulint i;
  
	prime = ut_find_prime(n);

	table = mem_alloc(sizeof(lock_hash_table_t));

	array = ut_malloc(sizeof(lock_hash_cell_t) * prime);

	table->array = array;
	table->n_cells = prime;
	table->n_mutexes = 0;
	table->mutexes = NULL;
	table->heaps = NULL;
	table->heap = NULL;
	table->magic_n = LOCK_HASH_TABLE_MAGIC_N;

	/* Initialize the cell array */

	for (i = 0; i < prime; i++) {    
		lock_hash_cell_t * cell = lock_hash_get_nth_cell(table, i);
		UT_LIST_INIT(*cell);
	}

	return(table);
}


lock_hash_cell_t * 
lock_hash_get_nth_cell(lock_hash_table_t * h, ulint n) {

	ut_ad(n < h->n_cells);
	return(h->array + n);

}


ulint
lock_hash_get_n_cells(lock_hash_table_t * h) {
	return(h->n_cells);
}



/**
 * [SREVILAK 2009-11-27].  Given space and page, return
 * head of (the left to right) lock hash chain 
 */
lock_hash_cell_t *
lock_hash_get_hash_cell(ulint space, ulint page_no) {
  
	/* nth_cell already has mod applied */
	const ulint nth_cell = lock_hash_calc_hash(lock_rec_fold(space, page_no),
						   lock_sys->lock_rec_hash);
	
	lock_hash_cell_t * chain = lock_hash_get_nth_cell(lock_sys->lock_rec_hash, 
							  nth_cell);

	return chain;
}


ulint
lock_hash_calc_hash(
/*===========*/
                    /* out: hashed value */
                    ulint		fold,	/* in: folded value */
                    lock_hash_table_t*	table)	/* in: hash table */
{
	return(ut_hash_ulint(fold, table->n_cells));
}



/*
 * Local Variables:
 * c-basic-offset: 8
 * indent-tabs-mode: t
 * End:
 */
