# $Id: umb_secondary_snapshot.inc,v 1.3 2009/11/18 22:47:43 srevilak Exp $
#
# Tests for snapshot reads on secondary indexes

set global innodb_debug_lock_manager = 1;

--echo NOTE: stating isolation level
select @@global.tx_isolation;

let $user = root;

--echo NOTE: setting up two connections
connect (con1,localhost,$user,,);
connect (con2,localhost,$user,,);

--echo NOTE con1 isolation level
connection con1;
select @@tx_isolation;

--echo NOTE con2 isolation level
connection con2;
select @@tx_isolation;

connection default;

##################################################################

--source include/umb_create_umb2.inc

--echo ====================== BEGIN CASE 1 ======================
--echo NOTE: insert not seen by scan across secondary index

--echo CON1
connection con1;
begin;
insert into umb2 values (25, 25, 0);

--echo CON2
connection con2;
begin;
select c2 from umb2 where c2 between 20 and 30;
commit;

--echo CON2, second transaction
begin;
select c2 from umb2 where c2 between 21 and 31;
commit;

--echo CON1 (sees own insert)
connection con1;
select * from umb2 where c1 = 25;
commit;

connection default;
--echo NOTE: table after test
begin;
select * from umb2;
commit;

--echo ====================== END CASE 1 ======================

##################################################################

--source include/umb_create_umb2.inc

--echo ====================== BEGIN CASE 2 ======================
--echo NOTE: repeated scan across insert sees same results

--echo CON1
connection con1;
begin;
select c2 from umb2 where c2 between 20 and 30;

--echo CON2
connection con2;
begin;
insert into umb2 values (25, 25, 0);

--echo CON1 repeats read (shouldn't see 25)
connection con1;
select c2 from umb2 where c2 between 19 and 30;

--echo CON2
connection con2;
commit;

--echo CON1 repeats again (shouldn't see 25)
connection con1;
select c2 from umb2 where c2 between 20 and 31;
commit;

connection default;
--echo NOTE: table after test (should see 25)
begin;
select * from umb2;
commit;

--echo ====================== END CASE 2 ======================


##################################################################

--source include/umb_create_umb2.inc

--echo ====================== BEGIN CASE 3 ======================
--echo NOTE: repeated scan across delete sees same results.

--echo CON1
connection con1;
begin;
select c2 from umb2 where c2 between 15 and 25;

--echo CON2
connection con2;
begin;
delete from umb2 where c1 = 20;

--echo CON1 (repeats read, should see 20);
connection con1;
select c2 from umb2 where c2 between 16 and 25;

--echo CON2
connection con2;
commit;

--echo CON1 (repeats read, should see 20)
connection con1;
select c2 from umb2 where c2 between 15 and 24;
commit;

connection default;
--echo NOTE: table after test (shouldn't see 20)
begin;
select * from umb2;
commit;

--echo ====================== END CASE 3 ======================



##################################################################

--source include/umb_create_umb2.inc

--echo ====================== BEGIN CASE 4 ======================
--echo NOTE: repeated scan across delete, delete first.

--echo CON1
connection con1;
begin;
delete from umb2 where c1 = 30;

--echo CON2 (sees 30)
connection con2;
begin;
select c2 from umb2 where c2 between 29 and 31;

--echo CON1
connection con1;
commit;

--echo CON2 (still sees 30)
connection con2;
select c2 from umb2 where c2 between 28 and 31;
commit;

connection default;
--echo NOTE: table after test (shouldn't see 30)
begin;
select * from umb2;
commit;

--echo ====================== END CASE 4 ======================


##################################################################

--source include/umb_create_umb2.inc

--echo ====================== BEGIN CASE 5 ======================
--echo NOTE: repeated scan across moved index entry

--echo CON1
connection con1;
begin;
select c2 from umb2 where c2 >= 20;

--echo CON2 (moves 30 -> 50)
connection con2;
begin;
update umb2 set c2 = 50 where c2 = 30;

--echo CON1 (still sees 20, 30, 40)
connection con1;
select c2 from umb2 where c2 >= 19;

--echo CON2 
connection con2;
commit;

--echo CON1 (still sees 20, 30, 40)
connection con1;
select c2 from umb2 where c2 >= 18;
commit;

connection default;
--echo NOTE: table after test (sees tuple (30, 50, 0))
begin;
select * from umb2;
commit;

--echo ====================== END CASE 5 ======================


##################################################################

--source include/umb_create_umb2.inc

--echo ====================== BEGIN CASE 6 ======================
--echo NOTE: repeated scan across moved index entry.  Move first

--echo CON1 (moves 30 -> 50)
connection con1; 
begin;
update umb2 set c2 = 50 where c2 = 30;

--echo CON2 (still sees 20, 30, 40)
connection con2;
begin;
select c2 from umb2 where c2 >= 20;

--echo CON1
connection con1;
commit;

--echo CON2 (still sees 20, 30, 40)
connection con2;
select c2 from umb2 where c2 >= 18;
commit;

connection default;
--echo NOTE: table after test (sees tuple (30, 50, 0))
begin;
select * from umb2;
commit;

--echo ====================== END CASE 6 ======================


##################################################################

--echo CLEANUP
disconnect con1;
disconnect con2;
--disable_warnings
drop table if exists umb2;
--enable_warnings


# Local Variables:
# mode: sql
# End:
