# $Id: umb_snapshot_read.inc,v 1.3 2009/11/18 22:47:43 srevilak Exp $
# $Source: /home/srevilak/Documents/schoolwork/phd/REPOSITORY/cvsroot/innobase-5.1.31-si/mysql-test/include/umb_snapshot_read.inc,v $
#
# Basic non-blocking snapshot read test.  Should produce the
# same results at "REPEATABLE READ" (aka, ordinary snapshot isolation)
# or SERIALIZABLE (aka, Serializable SI).


set global innodb_debug_lock_manager = 1;

--echo NOTE: stating isolation level
select @@global.tx_isolation;

let $user = root;

--echo NOTE: setting up two connections
connect (con1,localhost,$user,,);
connect (con2,localhost,$user,,);

--echo NOTE con1 isolation level
connection con1;
select @@tx_isolation;

--echo NOTE con2 isolation level
connection con2;
select @@tx_isolation;

connection default;

--source include/umb_create_umb1.inc

--echo ====================== BEGIN CASE 1a ======================
--echo NOTE: update not seen until commit

--echo CON1
connection con1;
begin;
select * from umb1 where pk = 3;

--echo CON2
connection con2;
begin;
update umb1 set c1 = 999 where pk = 3;

--echo CON1
connection con1;
select * from umb1 where pk = 3;
commit;

--echo CON2
connection con2;
commit;

--echo CON1 (new transaction)
begin;
select * from umb1 where pk = 3;
commit;

connection default;
--echo NOTE: table after test
select * from umb1;

--echo ====================== END CASE 1a ======================

##################################################################

--source include/umb_create_umb1.inc

--echo ====================== BEGIN CASE 1b ======================
--echo NOTE: delete not seen until commit

--echo CON1
connection con1;
begin;
select * from umb1 where pk between 3 and 5;

--echo CON2
connection con2;
begin;
delete from umb1 where pk = 4;

--echo CON1
connection con1;
select * from umb1 where pk between 3 and 5;
commit;

--echo CON2
connection con2;
commit;

--echo CON1 (new transaction)
begin;
select * from umb1 where pk between 3 and 5;
commit;

connection default;
--echo NOTE: table after test
select * from umb1;

--echo ====================== END CASE 1b ======================


##################################################################

--source include/umb_create_umb1.inc

--echo ====================== BEGIN CASE 1c ======================
--echo NOTE: insert not seen until commit

--echo CON1
connection con1;
begin;
select * from umb1 where pk >= 4;

--echo CON2
connection con2;
begin;
insert into umb1 (pk, c1) values (100, 100);

--echo CON1
connection con1;
select * from umb1 where pk >= 4;
commit;

--echo CON2
connection con2;
commit;

--echo CON1 (new transaction)
begin;
select * from umb1 where pk >= 4;
commit;

connection default;
--echo NOTE: table after test
select * from umb1;

--echo ====================== END CASE 1c ======================

##################################################################

--source include/umb_create_umb1.inc

--echo ====================== BEGIN CASE 2a ======================
--echo NOTE: long update not seen until commit

--echo CON1
connection con1;
begin;
select * from umb1 where pk > 1;

--echo CON2
connection con2;
begin;
update umb1 set c1 = 9999 where pk = 4;

--echo CON1
connection con1;
select * from umb1 where pk > 1;
commit;
--echo CON1 (new transaction)
begin;
select * from umb1 where pk > 1;
commit;

--echo CON2 (commit update)
connection con2;
commit;

--echo CON1 (new transaction)
begin;
select * from umb1 where pk > 1;
commit;

connection default;
--echo NOTE: table after test
select * from umb1;

--echo ====================== END CASE 2a ======================

##################################################################

--source include/umb_create_umb1.inc

--echo ====================== BEGIN CASE 2b ======================
--echo NOTE: long delete not seen until commit

--echo CON1
connection con1;
begin;
select * from umb1 where pk >= 3;

--echo CON2
connection con2;
begin;
delete from umb1 where pk >= 4;

--echo CON1
connection con1;
select * from umb1 where pk >= 3;
commit;

--echo CON1 (new transaction)
begin;
select * from umb1 where pk >= 3;
commit;

--echo CON2 (commit delete)
connection con2;
commit;

--echo CON1 (new transaction)
begin;
select * from umb1 where pk >= 3;
commit;

connection default;
--echo NOTE: table after test
select * from umb1;

--echo ====================== END CASE 2b ======================

##################################################################

--source include/umb_create_umb1.inc

--echo ====================== BEGIN CASE 2c ======================
--echo NOTE: long insert not seen until commit

--echo CON1
connection con1;
begin;
select * from umb1 where pk > 10;

--echo CON2
connection con2;
begin;
insert into umb1 (pk, c1) values (100, 100);

--echo CON1
connection con1;
select * from umb1 where pk > 10;
commit;

--echo CON1 (new transaction)
begin;
select * from umb1 where pk = 100;
commit;

--echo CON2 (commit insert)
connection con2;
commit;

--echo CON1 (new transaction)
begin;
select * from umb1 where pk > 20;
commit;

connection default;
--echo NOTE: table after test
select * from umb1;

--echo ====================== END CASE 2c ======================

##################################################################

--source include/umb_create_umb1.inc

--echo ====================== BEGIN CASE 3a ======================
--echo NOTE: update first, not seen until commit

--echo CON1
connection con1;
begin;
update umb1 set c1 = 9999 where pk = 5;

--echo CON2
connection con2;
begin;
select * from umb1 where pk between 4 and 6;
commit;

--echo CON2
connection con2;
begin;
select * from umb1 where pk between 4 and 6;
commit;

--echo CON1 (commit update)
connection con1;
commit;

--echo CON2 (new transaction)
connection con2;
begin;
select * from umb1 where pk between 4 and 6;
commit;

connection default;
--echo NOTE: table after test
select * from umb1;

--echo ====================== END CASE 3a ======================

##################################################################

--source include/umb_create_umb1.inc

--echo ====================== BEGIN CASE 3b ======================
--echo NOTE: delete first, not seen until commit

--echo CON1
connection con1;
begin;
delete from umb1 where pk = 5;

--echo CON2
connection con2;
begin;
select * from umb1 where pk between 4 and 6;
commit;

--echo CON2
connection con2;
begin;
select * from umb1 where pk between 4 and 6;
commit;

--echo CON1 (commit delete)
connection con1;
commit;

--echo CON2 (new transaction)
connection con2;
begin;
select * from umb1 where pk between 4 and 6;
commit;

connection default;
--echo NOTE: table after test
select * from umb1;

--echo ====================== END CASE 3b ======================

##################################################################

--source include/umb_create_umb1.inc

--echo ====================== BEGIN CASE 3c ======================
--echo NOTE: insert first, not seen until commit

--echo CON1
connection con1;
begin;
delete from umb1 where pk = 4;
insert into umb1 (pk, c1) values (4, 8888);

--echo CON2
connection con2;
begin;
select * from umb1 where pk between 3 and 5;
commit;

--echo CON2
connection con2;
begin;
select * from umb1 where pk between 3 and 5;
commit;

--echo CON1 (commit delete)
connection con1;
commit;

--echo CON2 (new transaction)
connection con2;
begin;
select * from umb1 where pk between 3 and 5;
commit;

connection default;
--echo NOTE: table after test
select * from umb1;

--echo ====================== END CASE 3c ======================

##################################################################

--echo CLEANUP

disconnect con1;
disconnect con2;

--disable_warnings
drop table if exists umb1;
--enable_warnings


# Local Variables:
# mode: sql
# End:
