# $Id: umb_write_write_conflict.inc,v 1.3 2009/12/10 04:12:46 srevilak Exp $
#
# Common tests for write write conflicts.
#
# PRE_REQUISITES: you have declared two connections, con1 and con2.
# POST_REQUISITES: you cleanup your connections

# make sure we have the right strategy
show variables like 'innodb_write_write_conflict_strategy';

let $strategy = query_get_value("show variables like 'innodb_write_write_conflict_strategy'", Value, 1); 

# Figure out which write/write conflict strategy we are using.
# This tells us what errors to expect.

let $using_first_updater = 0;
let $using_second_updater = 0;
let $using_other = 0;

if (`SELECT $strategy = 0`) {
  let $using_first_updater = 1;
}

if (`SELECT $strategy = 1`) {
  let $using_second_updater = 1;
}

if (`SELECT $strategy > 1`) {
  let $using_other = 1;
}

if ($using_other) {
  die "unhandled write/write conflict strategy";
}

eval select $strategy as 'strategy', 
  $using_first_updater as 'using_first_updater', 
  $using_second_updater as 'using_second_updater',
  $using_other as 'using_other';


--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 1 =========
--echo NOTE: update/update conflict 
connection con1;
--echo "NOTE: con1"
begin;
update umb1 set c1 = 0 where pk = 4;

connection con2;
--echo "NOTE: con2"
begin;
if ($using_first_updater) {
  --error ER_LOCK_WAIT_TIMEOUT
  update umb1 set c1 = 0 where pk = 4;
}
if ($using_second_updater) {
  --error ER_GET_ERRMSG
  update umb1 set c1 = 0 where pk = 4;
}
rollback;

connection con1;
--echo "NOTE: con1"
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 1 ===========

##################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 2 =========
--echo NOTE: delete/delete conflict 

--echo NOTE: con1
connection con1;
begin;
delete from umb1 where pk = 5;

--echo NOTE: con2
connection con2;
begin;
if ($using_first_updater) {
  --error ER_LOCK_WAIT_TIMEOUT
  delete from umb1 where pk = 5;
}
if ($using_second_updater) {
  --error ER_GET_ERRMSG
  delete from umb1 where pk = 5;
}
rollback;

--echo NOTE: con1
connection con1;
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 2 =========


##################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 3 =========
--echo NOTE: update/delete conflict 

--echo NOTE: con1
connection con1;
begin;
update umb1 set c1 = 99 where pk = 4;

--echo NOTE: con2
connection con2;
begin;
if ($using_first_updater) {
  --error ER_LOCK_WAIT_TIMEOUT
  delete from umb1 where pk = 4;
}
if ($using_second_updater) {
  --error ER_GET_ERRMSG
  delete from umb1 where pk = 4;
}
rollback;

--echo NOTE: con1
connection con1;
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 3 =========

##################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 4 =========
--echo NOTE: delete/update conflict 

--echo NOTE: con1
connection con1;
begin;
delete from umb1 where pk = 3;

--echo NOTE: con2
connection con2;
begin;
if ($using_first_updater) {
  --error ER_LOCK_WAIT_TIMEOUT
  update umb1 set c1 = 999 where pk = 3;
}
if ($using_second_updater) {
  --error ER_GET_ERRMSG
  update umb1 set c1 = 999 where pk = 3;
}
rollback;

--echo NOTE: con1
connection con1;
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 4 =========

##################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 5 =========
--echo NOTE: insert/insert conflict 

--echo NOTE: con1
connection con1;
begin;
insert into umb1 (pk, c1) values (100, 100);

--echo NOTE: con2
connection con2;
begin;
--error ER_DUP_ENTRY
insert into umb1 (pk, c1) values (100, 100);
rollback;

--echo NOTE: con1
connection con1;
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 5 =========

##################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 6 =========
--echo NOTE: update/update conflict, different access paths

--echo NOTE: con1
connection con1;
begin;
update umb1 set c1 = 9999 where pk = 3;

--echo NOTE: con2
connection con2;
begin;
if ($using_first_updater) {
  --error ER_LOCK_WAIT_TIMEOUT
  update umb1 set c1 = 8888 where c1 = 9;
}
if ($using_second_updater) {
  --error ER_GET_ERRMSG
  update umb1 set c1 = 8888 where c1 = 9;
}
rollback;

--echo NOTE: con1
connection con1;
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 6 =========

##################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 7 =========
--echo NOTE: update followed by update.  Positive test case

--echo NOTE: con1
connection con1;
begin;
update umb1 set c1 = 8888 where pk = 3;
commit;

--echo NOTE: con2
connection con2;
begin;
select * from umb1;
update umb1 set c1 = 7777 where pk = 3;
commit;


--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 7 =========

##################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 8 =========
--echo NOTE: update followed by delete.  Positive test case

--echo NOTE: con1
connection con1;
begin;
update umb1 set c1 = 8888 where pk = 5;
commit;

--echo NOTE: con2
connection con2;
begin;
select * from umb1 where pk = 5;
delete from umb1 where pk = 5;
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 8 =========

##################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 9 =========
--echo NOTE: delete followed by update.  Positive test case

--echo NOTE: con1
connection con1;
begin;
delete from umb1 where pk = 4;
commit;

--echo NOTE: con2
connection con2;
begin;
update umb1 set c1 = 99999 where pk = 4;
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 9 =========

##################################################################


--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 10 =========
--echo NOTE: delete followed by delete.  Positive test case

--echo NOTE: con1
connection con1;
begin;
delete from umb1 where pk = 1;
commit;

--echo NOTE: con2
connection con2;
begin;
delete from umb1 where pk = 1;
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 10 =========


################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 11 =========
--echo NOTE: Delete x + concurrent insert of x

-- use serializable; in debug builds, this forces an additional
-- concurrent write/write conflict test in the lock manager.

--echo NOTE: con1
connection con1;
set transaction isolation level serializable;
begin;
delete from umb1 where pk = 3;

--echo NOTE: con2
connection con2;
set transaction isolation level serializable;
begin;
send insert into umb1 (pk, c1) values (3, 9999);

--echo NOTE: con1
connection con1;
sleep 0.5;
commit;

--echo NOTE: con2
connection con2;
--error ER_GET_ERRMSG
reap;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 11 =========


################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 12 =========
--echo NOTE: postive case: delete x + later insert of x

--echo NOTE: con1
connection con1;
begin;
delete from umb1 where pk = 3;
commit;

--echo NOTE: con2
connection con2;
begin;
insert into umb1 (pk, c1) values (3, 9999);
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 12 =========

################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 13 =========
--echo NOTE: insert x + concurrent delete of c

--echo NOTE: con1
connection con1;
begin;
insert into umb1 (pk, c1) values (10, 9999);

--echo NOTE: con2
connection con2;
begin;
send delete from umb1 where pk = 10;

--echo NOTE: con1
connection con1;
sleep 0.5;
commit;

--echo NOTE: con2
connection con2;
--error ER_GET_ERRMSG
reap;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 13 =========

################################################################

--source include/umb_create_umb1.inc

--echo ======== BEGIN CASE 14 =========
--echo NOTE: positive case insert x + non-concurrent delete of c

--echo NOTE: con1
connection con1;
begin;
insert into umb1 (pk, c1) values (10, 9999);
commit;

--echo NOTE: con2
connection con2;
begin;
delete from umb1 where pk = 10;
commit;

--echo table after test
connection default;
select * from umb1;

--echo ======== END CASE 14 =========


# Local Variables:
# mode: sql
# End:
