/*
 * $Id: umb-test-malloc-mem-heap.c,v 1.2 2010/02/23 02:47:08 srevilak Exp $
 * 
 * A quick performance comparison of malloc, vs mem_heap_t allocation
 */

#ifdef UNIV_DEBUG

#include <sys/time.h>
#include "umb-test.h"
#include "mem0mem.h"

/*
 * A 64-byte chunk of data
 */
typedef struct {
  char data[64];
} A;

#define ITERATIONS 50
#define DATA_SIZE 5000
static A * data[DATA_SIZE];

/*
 * Given two struct timevals, compute
 * the difference in seconds
 */
static double tv_delta(const struct timeval * start, 
                       const struct timeval * end) {
  double delta_sec = end->tv_sec - start->tv_sec;
  double delta_usec = end->tv_usec - start->tv_usec;
  
  delta_sec += (delta_usec / 1000000.0);

  return delta_sec;
}


static double try_with_mem_heap(mem_heap_t * heap) {
  ulint iter;
  struct timeval t0;
  struct timeval t1;

  
  gettimeofday(&t0, NULL);
  for (iter = 0; iter < DATA_SIZE; iter++) {
    data[iter] = mem_heap_alloc(heap, sizeof(A));
  }
  mem_heap_empty(heap);

  gettimeofday(&t1, NULL);
  return tv_delta(&t0, &t1);
}


static double try_with_malloc(void) {

  ulint iter;  
  struct timeval t0;
  struct timeval t1;

  gettimeofday(&t0, NULL);
  for (iter = 0; iter < DATA_SIZE; iter++) {
    data[iter] = malloc(sizeof(A));
  }
  for (iter = 0; iter < DATA_SIZE; iter++) {
    free(data[iter]);
  }
  gettimeofday(&t1, NULL);
  
  return tv_delta(&t0, &t1);
}


static void test_with_mem_heap(void) {
  mem_heap_t * heap = mem_heap_create(256);

  ulint iter;
  double cumsum = 0.0;
  for (iter = 0; iter < ITERATIONS; iter++) {
    double this_run = try_with_mem_heap(heap);
    cumsum += this_run;
  }
  fprintf(stderr, "mem_pool: %.03f sec\n", cumsum);
  
  mem_heap_free(heap);
}



static void test_with_malloc(void) {
  UMB_TEST_FUNC_ANNOUNCE("test_with_malloc");

  ulint iter;
  double cumsum = 0.0;
  for (iter = 0; iter < ITERATIONS; iter++) {
    double this_run = try_with_malloc();
    cumsum += this_run;
  }
  fprintf(stderr, "malloc: %.03f sec\n", cumsum);
}



UMB_TEST_ENTRY void umb_test_malloc_mem_heap(void) {
  UMB_TEST_ANNOUNCE;
  
  test_with_malloc();
  test_with_mem_heap();

}


#endif /* UNIV_DEBUG */
